# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.4.rc2'
date = '2024-07-04'
banner = 'SageMath version 10.4.rc2, Release Date: 2024-07-04'
